#include "PainterEngine_Application.h"
#include "pearl_white_story.h"


PX_Application App;
px_void PX_MidiCallbackTrigger(struct _PX_Midi* midi, px_byte opcode, px_byte  track, px_byte  instrument, px_byte Note, px_byte velocity, px_void* userptr)
{
	px_char content[8];
	PX_Application* pApp = (PX_Application*)userptr;
	PX_Object_Piano* pPiano = PX_ObjectGetDesc(PX_Object_Piano, pApp->piano_keyboard);
	if (opcode==0x90)
	{
		px_int index;
		PX_MidiNoteToString(Note, content);
		index = PX_PianoKeyNameToIndex(content);
		PX_PianoTriggerIndex(&pApp->Piano, index, velocity / 127.0f);
		pPiano->keyState[index] = PX_OBJECT_PIANO_KEYSTATE_DOWN;
	}

	if (opcode == 0x80)
	{
		px_int index;
		PX_MidiNoteToString(Note, content);
		index = PX_PianoKeyNameToIndex(content);
		PX_PianoTriggerIndex(&pApp->Piano, index, 0);
		pPiano->keyState[index] = PX_OBJECT_PIANO_KEYSTATE_STANDBY;
	}
}

px_void PX_ApplicationOnPianoKey(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_PianoTriggerIndex(&pApp->Piano, PX_Object_Event_GetIndex(e), 1);
}

px_void PX_ApplicationOnPianoPlay(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_MidiPlay(&pApp->midi);
}

px_void PX_ApplicationOnPianoStop(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_MidiPause(&pApp->midi);
}


px_void PX_PianoRead(px_void* userptr, px_byte* pBuffer, px_int readSize)
{
	if (!readSize||readSize>2048*2*2)
	{
		PX_memset(pBuffer, 0, readSize);
	}
	else
	{
		px_int oft = 0;
		PX_Application* pApp = (PX_Application*)userptr;
		px_int count = readSize / 4;//2bytes 2channels
		px_float out44100f[2048];
		px_int i;
		px_short* PCM = (px_short*)pBuffer;
		PX_PianoGo(&pApp->Piano, out44100f, count);
		for (i=0;i<count;i++)
		{
			PCM[oft++] = (px_short)(out44100f[i]*32768*22);
			PCM[oft++] = (px_short)(out44100f[i] * 32768*22);
		}
	}
}


px_bool PX_ApplicationInitialize(PX_Application *pApp,px_int screen_width,px_int screen_height)
{

	PX_ApplicationInitializeDefault(&pApp->runtime, screen_width, screen_height);
	PX_MidiInitialize(&pApp->midi, &pApp->runtime.mp_resources, PX_MidiCallbackTrigger, pApp);
	PX_MidiLoad(&pApp->midi, pearl_white_story, sizeof(pearl_white_story));
	PX_MidiSetQuarterNoteDuration(&pApp->midi, 960);
	PX_PianoInitialize( &pApp->runtime.mp_game, &pApp->Piano);
	PX_SoundPlayInitialize(&pApp->runtime.mp_game, &pApp->runtime.soundplay);
	PX_AudioInitialize(&pApp->runtime.soundplay);
	PX_SoundPlaySetUserRead(&pApp->runtime.soundplay, PX_PianoRead, pApp);
	pApp->root = PX_ObjectCreate(&pApp->runtime.mp_ui, 0, 0, 0, 0, 0, 0, 0);
	pApp->piano_keyboard = PX_Object_PianoCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, 52*16, 64);
	pApp->button_play = PX_Object_PushButtonCreate(&pApp->runtime.mp_ui, pApp->root, 5, 5, 64, 32, "Play", PX_NULL);
	pApp->button_stop= PX_Object_PushButtonCreate(&pApp->runtime.mp_ui, pApp->root, 84, 5, 64, 32, "Stop", PX_NULL);
	PX_ObjectRegisterEvent(pApp->piano_keyboard, PX_OBJECT_EVENT_VALUECHANGED, PX_ApplicationOnPianoKey, pApp);
	PX_ObjectRegisterEvent(pApp->button_play, PX_OBJECT_EVENT_EXECUTE, PX_ApplicationOnPianoPlay, pApp);
	PX_ObjectRegisterEvent(pApp->button_stop, PX_OBJECT_EVENT_EXECUTE, PX_ApplicationOnPianoStop, pApp);
	return PX_TRUE;
}

px_void PX_ApplicationUpdate(PX_Application *pApp,px_dword elapsed)
{
	PX_MidiUpdate(&pApp->midi, elapsed);
	
}

px_void PX_ApplicationRender(PX_Application *pApp,px_dword elapsed)
{
	px_surface *pRenderSurface=&pApp->runtime.RenderSurface;
	PX_RuntimeRenderClear(&pApp->runtime,PX_OBJECT_UI_DEFAULT_BACKGROUNDCOLOR);
	pApp->piano_keyboard->Width = pRenderSurface->width * 9 / 10.f;
	pApp->piano_keyboard->Height = pRenderSurface->height /5.f;
	pApp->piano_keyboard->x = pRenderSurface->width / 2 - pApp->piano_keyboard->Width / 2;
	pApp->piano_keyboard->y = pRenderSurface->height / 2 - pApp->piano_keyboard->Height / 2;
	PX_ObjectRender(pRenderSurface, pApp->root, elapsed);
}

px_void PX_ApplicationPostEvent(PX_Application *pApp,PX_Object_Event e)
{
	PX_ApplicationEventDefault(&pApp->runtime, e);
	PX_ObjectPostEvent( pApp->root, e);
}

